@@ -27,8 +27,8 @@ from typing import Iterable
 
 from . import db
 from .data_repository import generate_ars_mapping, generate_mac_codes, generate_alleles_and_xx_codes, \
-    generate_serology_mapping
-from .db import is_valid_mac_code, mac_code_to_alleles
+    generate_serology_mapping, generate_v2_to_v3_mapping
+from .db import is_valid_mac_code, mac_code_to_alleles, v2_to_v3_allele
 from .smart_sort import smart_sort_comparator
 
 HLA_regex = re.compile("^HLA-")
@@ -66,6 +66,8 @@ class ARD(object):
         self.dup_g, self._G, self._lg, self._lgx = generate_ars_mapping(self.db_connection, imgt_version)
         # Load Serology mappings
         generate_serology_mapping(self.db_connection, imgt_version)
+        # Load V2 to V3 mappings
+        generate_v2_to_v3_mapping(self.db_connection, imgt_version)
 
         # Close the current read-write db connection
         self.db_connection.close()
@@ -172,46 +174,66 @@ class ARD(object):
             return "/".join(sorted(set([self.redux_gl(a, redux_type) for a in glstring.split("/")]),
                                    key=functools.cmp_to_key(smart_sort_comparator)))
 
+        # Handle V2 to V3 mapping
+        if self.is_v2(glstring):
+            glstring = self._map_v2_to_v3(glstring)
+            return self.redux_gl(glstring, redux_type)
+
         # Handle Serology
         if self.is_serology(glstring):
-            if HLA_regex.search(glstring):
-                # Remove HLA- prefix
-                serology = glstring.split("-")[1]
-                alleles = self._get_alleles_from_serology(serology)
-                alleles = ['HLA-' + a for a in alleles]
-            else:
-                alleles = self._get_alleles_from_serology(glstring)
+            alleles = self._get_alleles_from_serology(glstring)
             return self.redux_gl("/".join(alleles), redux_type)
 
         loc_allele = glstring.split(":")
-        loc_name, code = loc_allele[0], loc_allele[1]
+        loc_antigen, code = loc_allele[0], loc_allele[1]
 
         # Handle XX codes
-        if (self.is_mac(glstring) and glstring.split(":")[1] == "XX") and loc_name in self.xx_codes:
-            return self.redux_gl("/".join(self.xx_codes[loc_name]), redux_type)
+        if self.is_XX(glstring, loc_antigen, code):
+            return self.redux_gl("/".join(self.xx_codes[loc_antigen]), redux_type)
 
         # Handle MAC
         if self.is_mac(glstring) and is_valid_mac_code(self.db_connection, code):
             if HLA_regex.search(glstring):
                 # Remove HLA- prefix
                 allele_name = glstring.split("-")[1]
-                loc_name, code = allele_name.split(":")
-                alleles = self._get_alleles(code, loc_name)
+                loc_antigen, code = allele_name.split(":")
+                alleles = self._get_alleles(code, loc_antigen)
                 alleles = ["HLA-" + a for a in alleles]
             else:
-                alleles = self._get_alleles(code, loc_name)
+                alleles = self._get_alleles(code, loc_antigen)
             return self.redux_gl("/".join(alleles), redux_type)
 
         return self.redux(glstring, redux_type)
 
+    def is_XX(self, glstring: str, loc_antigen: str = None, code: str = None) -> bool:
+        if loc_antigen is None or code is None:
+            loc_allele = glstring.split(":")
+            loc_antigen, code = loc_allele[0], loc_allele[1]
+        return self.is_mac(glstring) and code == "XX" and loc_antigen in self.xx_codes
+
     @staticmethod
     def is_serology(allele: str) -> bool:
         """
-        An allele is serology if the allele name after * is numeral only, no ':'
+        A serology has the locus name (first 2 letters for DRB1, DRB3, DQB1, DQA1, DPB1 and DPA1)
+        of the allele followed by numerical antigen.
+
         :param allele: The allele to test for serology
         :return: True if serology
         """
-        return allele.split('*')[1].isdigit()
+        if '*' in allele or ':' in allele:
+            return False
+
+        locus = allele[0:2]
+        if locus in ['DR', 'DP', 'DQ']:
+            antigen = allele[2:]
+            return antigen.isdigit()
+
+        locus = allele[0:1]
+        if locus in ['A', 'B', 'C', 'D']:
+            antigen = allele[1:]
+            return antigen.isdigit()
+
+        return False
 
     @staticmethod
     def is_mac(gl: str) -> bool:
@@ -223,28 +245,104 @@ class ARD(object):
         """
         return re.search(r":\D+", gl) is not None
 
+    @staticmethod
+    def is_v2(allele: str) -> bool:
+        """
+        Version 2 of the nomenclature is a single field.
+        It does not have any ':' field separator.
+        Eg: A*0104
+        :param allele: Possible allele
+        :return: Is the allele in V2 nomenclature
+        """
+        return '*' in allele and ':' not in allele
+
     def _is_valid_allele(self, allele):
         """
         Test if allele is valid in the current imgt database
         :param allele: Allele to test
         :return: bool to indicate if allele is valid
         """
-        return allele in self.valid_alleles
+        if self._remove_invalid:
+            return allele in self.valid_alleles
+        return True
 
-    def _get_alleles(self, code, loc_name) -> Iterable[str]:
+    def _get_alleles(self, code, locus_antigen) -> Iterable[str]:
         """
         Look up allele code in database and generate alleles
         :param code: allele code to look up
-        :param loc_name: locus name for alleles
+        :param locus_antigen: locus name for alleles
         :return: valid alleles corresponding to allele code
         """
         alleles = mac_code_to_alleles(self.db_connection, code)
-        return filter(self._is_valid_allele,
-                      [f'{loc_name}:{a}' for a in alleles])
+
+        # It's an allelic expansion if any of the alleles have a `:`
+        # else it's a group expansion
+        is_allelic_expansion = any([':' in allele for allele in alleles])
+        if is_allelic_expansion:
+            locus = locus_antigen.split('*')[0]  # Just keep the locus name
+            alleles = [f'{locus}*{a}' for a in alleles]
+        else:
+            alleles = [f'{locus_antigen}:{a}' for a in alleles]
+
+        if self._remove_invalid:
+            return filter(self._is_valid_allele, alleles)
+        else:
+            return alleles
 
     def _get_alleles_from_serology(self, serology) -> Iterable[str]:
         alleles = db.serology_to_alleles(self.db_connection, serology)
-        return filter(self._is_valid_allele, alleles)
+        if self._remove_invalid:
+            return filter(self._is_valid_allele, alleles)
+        else:
+            return alleles
+
+    def _combine_with_colon(self, digits_field):
+        num_of_digits = len(digits_field)
+        return ':'.join(digits_field[i:i + 2] for i in range(0, num_of_digits, 2))
+
+    def _predict_v3(self, v2_allele: str) -> str:
+        """
+        Use heuristic to predict V3 from V2
+
+        :param v2_allele: Allele in V2 format
+        :return: V3 format of V2 allele
+        """
+        # Separate out the locus and the allele name part
+        locus, allele_name = v2_allele.split('*')
+        # Separate out the numeric and non-numeric components
+        components = re.findall(r'^(\d+)(.*)', allele_name)
+        if not components:
+            return v2_allele
+        digits_field, non_digits_field = components.pop()
+        # final_allele is the result of the transformation
+        final_allele = digits_field
+        num_of_digits = len(digits_field)
+        if num_of_digits == 1:
+            return v2_allele
+        if num_of_digits > 2:
+            if locus.startswith('DP') and num_of_digits == 5:  # covers DPs with 5 digits
+                final_allele = digits_field[:3] + ':' + (digits_field[3:]) + non_digits_field
+            elif num_of_digits % 2 == 0:  # covers digits with 2, 4, 6, 8
+                final_allele = self._combine_with_colon(digits_field) + non_digits_field
+            else:
+                final_allele = digits_field[:2] + ':' + (digits_field[2:]) + non_digits_field
+        else:
+            if non_digits_field:
+                final_allele = digits_field + ':' + non_digits_field
+        return locus + '*' + final_allele
+
+    def _map_v2_to_v3(self, v2_allele):
+        """
+        Get V3 version of V2 versioned allele
+        :param v2_allele: V2 versioned allele
+        :return: V3 versioned allele
+        """
+        # Check if it's in the exception case mapping
+        v3_allele = v2_to_v3_allele(self.db_connection, v2_allele)
+        if not v3_allele:
+            # Try and predict V3
+            v3_allele = self._predict_v3(v2_allele)
+        return v3_allele
 
     def isvalid(self, allele: str) -> bool:
         """
@@ -255,7 +353,11 @@ class ARD(object):
         :return: allele or empty
         :rtype: bool
         """
-        if not self.is_mac(allele) and not self.is_serology(allele):
+        if allele == '':
+            return False
+        if not self.is_mac(allele) and \
+                not self.is_serology(allele) and \
+                not self.is_v2(allele):
             # Alleles ending with P or G are valid_alleles
             if allele.endswith(('P', 'G')):
                 # remove the last character
@@ -300,17 +402,16 @@ class ARD(object):
         :return: ARS reduced allele
         :rtype: str
         """
-        loc_name, code = allele.split(":")
+        locus_antigen, code = allele.split(":")
         if HLA_regex.search(allele):
-            loc_name = loc_name.split("-")[1]
+            locus_antigen = locus_antigen.split("-")[1]  # Remove HLA- prefix
         if is_valid_mac_code(self.db_connection, code):
-            alleles = self._get_alleles(code, loc_name)
+            alleles = self._get_alleles(code, locus_antigen)
             group = [self.toG(a) for a in alleles]
             if "X" in group:
                 return ''
             else:
                 return "/".join(group)
-
         else:
             return ''
 
@@ -340,12 +441,23 @@ class ARD(object):
         :return: List
         :rtype: List
         """
-        loc_name, code = mac_code.split(":")
+        locus_antigen, code = mac_code.split(":")
         if is_valid_mac_code(self.db_connection, code):
             if HLA_regex.search(mac_code):
-                loc_name = loc_name.split("-")[1]
-                return ['HLA-' + a for a in self._get_alleles(code, loc_name)]
+                locus_antigen = locus_antigen.split("-")[1]  # Remove HLA- prefix
+                return ['HLA-' + a for a in self._get_alleles(code, locus_antigen)]
             else:
-                return list(self._get_alleles(code, loc_name))
+                return list(self._get_alleles(code, locus_antigen))
 
         return ''
+
+    def v2_to_v3(self, v2_allele) -> str:
+        """
+        Convert Version 2 Allele Name to Version 3 Allele Name
+
+        :param v2_allele: Version 2 Allele Name
+        :return: Version 3 Allele Name
+        """
+        if self.is_v2(v2_allele):
+            return self._map_v2_to_v3(v2_allele)
+        return v2_allele
