@@ -25,6 +25,7 @@ import functools
 import re
 
 expr_regex = re.compile('[NQLSGg]')
+glstring_chars = re.compile('[/|+^~]')
 
 
 @functools.lru_cache(maxsize=1000)
@@ -43,6 +44,13 @@ def smart_sort_comparator(a1, a2):
     if a1 == a2:
         return 0
 
+    # GL String matches
+    if re.search(glstring_chars, a1) or re.search(glstring_chars, a2):
+        if a1 > a2:
+            return 1
+        else:
+            return -1
+
     # remove any non-numerics
     a1 = re.sub(expr_regex, '', a1)
     a2 = re.sub(expr_regex, '', a2)
@@ -92,4 +100,3 @@ def smart_sort_comparator(a1, a2):
 
     # All fields are considered equal after 4th field
     return 0
-
